@@ -30,6 +30,7 @@
 #'    \item{all_logliks}{The log-likelihood of the estimates from all estimation rounds, if applicable.}
 #'    \item{which_converged}{Indicators of which estimation rounds converged, if applicable.}
 #'    \item{which_round}{Indicators of which round of optimization each estimate belongs to, if applicable.}
+#'    \item{seeds}{The seeds used in the estimation in \code{fitSTVAR}, if applicable.}
 #'    \item{LS_estimates}{The least squares estimates of the parameters in the form
 #'      \eqn{(\phi_{1,0},...,\phi_{M,0},\varphi_1,...,\varphi_M,\alpha} (intercepts replaced by unconditional means
 #'      if mean parametrization is used), if applicable.}
@@ -44,7 +45,7 @@
 #'      \emph{Journal of Econometrics}, \strong{84}:1, 1-36.
 #'    \item Hubrich K., Teräsvirta. T. 2013. Thresholds and Smooth Transitions in Vector Autoregressive Models.
 #'      \emph{CREATES Research Paper 2013-18, Aarhus University.}
-#'    \item Lanne M., Virolainen S. 2024. A Gaussian smooth transition vector autoregressive model:
+#'    \item Lanne M., Virolainen S. 2025. A Gaussian smooth transition vector autoregressive model:
 #'       An application to the macroeconomic effects of severe weather shocks. Unpublished working
 #'       paper, available as arXiv:2403.14216.
 #'    \item Kheifets I.L., Saikkonen P.J. 2020. Stationarity and ergodicity of Vector STAR models.
@@ -53,7 +54,7 @@
 #'      \emph{Journal of Economic Dynamics & Control}, \strong{84}, 43-57.
 #'    \item Tsay R. 1998. Testing and Modeling Multivariate Threshold Models.
 #'      \emph{Journal of the American Statistical Association}, \strong{93}:443, 1188-1202.
-#'    \item Virolainen S. 2024. Identification by non-Gaussianity in structural threshold and
+#'    \item Virolainen S. 2025. Identification by non-Gaussianity in structural threshold and
 #'       smooth transition vector autoregressive models. Unpublished working
 #'       paper, available as arXiv:2404.19707.
 #'  }
@@ -401,6 +402,7 @@ alt_stvar <- function(stvar, which_largest=1, which_round, calc_std_errors=FALSE
   ret$all_estimates <- stvar$all_estimates
   ret$all_logliks <- stvar$all_logliks
   ret$which_converged <- stvar$which_converged
+  ret$seeds <- stvar$seeds
   if(!is.null(stvar$which_round)) {
     ret$which_round <- which_round
   }
@@ -1086,6 +1088,7 @@ filter_estimates <- function(stvar, which_largest=1, filter_stab=TRUE, calc_std_
   ret$all_estimates <- all_estimates
   ret$all_logliks <- all_logliks
   ret$which_converged <- stvar$which_converged
+  ret$seeds <- stvar$seeds
   if(!is.null(stvar$which_round)) {
     ret$which_round <- which_round
   }
